﻿using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace AnimationClassLibrary
{
    /// <summary>
    /// 动画处理的辅助类。
    /// </summary>
    public class AnimationHelper
    {
        /// <summary>
        /// 将模型的初始变换矩阵附加到每个关键帧上。
        /// </summary>
        /// <param name="animationData">要处理的动画数据，类型为AnimationData</param>
        /// <param name="originalModelTransform">模型的初始变换矩阵</param>
        public static void AddModelTransform(AnimationData animationData,Matrix[] originalModelTransform)
        {
            foreach (KeyValuePair<string, AnimationClip> animation in animationData.AnimationClips)
            {
                ProcessAnimationClip(animation.Value, originalModelTransform);
            }
        }

        private static void ProcessAnimationClip(AnimationClip animationClip, Matrix[] originalModelTransform)
        {
            // 遍历动画片段的每个关键帧
            foreach (Keyframe keyframe in animationClip.Keyframes)
            {
                // 获取关键帧中的BoneIndex数据
                int boneIndex = keyframe.BoneIndex;
                // 获取关键帧中的位置、缩放、旋转的数据
                Vector3 position = keyframe.Position;
                Vector3 scale = keyframe.Scale;
                Quaternion rotation = keyframe.Rotation;
                // 根据以上信息还原变换矩阵
                Matrix originalTransform = Matrix.CreateScale(scale) *
                    Matrix.CreateFromQuaternion(rotation) * Matrix.CreateTranslation(position);
                // 附加模型的原始变换矩阵生成经过处理的变换矩阵
                Matrix newTransform = originalTransform * originalModelTransform[boneIndex];
                // 将这个新矩阵还原为缩放、旋转、位置信息
                newTransform.Decompose(out scale, out rotation, out position);
                // 将新的信息写入当前处理的关键帧中
                keyframe.Position = position;
                keyframe.Rotation = rotation;
                keyframe.Scale = scale;
            }
        }

        /// <summary>
        /// 将模型的初始变换矩阵附加到每个关键帧上。
        /// </summary>
        /// <param name="animationData">要处理的动画数据，类型为ModelAnimationData</param>
        /// <param name="originalModelTransform">模型的初始变换矩阵</param>
        public static void AddModelTransform(ModelAnimationData animationData, Matrix[] originalModelTransform)
        {
            foreach (KeyValuePair<string, ModelAnimationClip> animation in animationData.ModelAnimationClips)
            {
                ProcessModelAnimationClip(animation.Value, originalModelTransform);
            }
        }

        private static void ProcessModelAnimationClip(ModelAnimationClip animationClip, Matrix[] originalModelTransform)
        {
            // 遍历动画片段的每个关键帧
            foreach (ModelKeyframe keyframe in animationClip.Keyframes)
            {
               // 附加模型的原始变换矩阵生成经过处理的变换矩阵
                Matrix newTransform = keyframe.Transform * originalModelTransform[keyframe.BoneIndex];
                keyframe.Transform = newTransform;
            }
        }
    }
}
